#include <stdlib.h>
#include <string.h>
#include "plateTime.h"

#define SEPARATOR ';'

int readField(char buffer[], char sep, FILE *f);

PlateTimeList addPlateTime(PlateTimeList list, PlateTime data)
{
    PlateTimeList listElement = (PlateTimeList)malloc(sizeof(PlateTimeElement));
    listElement->next = list;
    listElement->data = data;
    return listElement;
}

PlateTimeList concatPlateTimeLists(PlateTimeList head, PlateTimeList tail)
{
    if (head == NULL)
    {
        return tail;
    }
    else if (tail == NULL)
    {
        return NULL;
    }
    else
    {
        PlateTimeList tmp = head;
        while (tmp->next != NULL)
            tmp = tmp->next;

        tmp->next = tail;
    }
    return head;
}

int getPlateTimeListLength(PlateTimeList list)
{
    int length = 0;
    while (list != NULL)
    {
        length++;
        list = list->next;
    }
    return length;
}

PlateTime* findPlateTime(PlateTimeList list, Plate plate)
{
    while (list != NULL)
    {
        if (strcmp(list->data.plate, plate) == 0)
        {
            return &(list->data);
        }
        list = list->next;
    }
    return NULL;
}

void destroyPlateTimeList(PlateTimeList list)
{
    if (list != NULL)
    {
        destroyPlateTimeList(list->next);
        free(list);
    }
}

PlateTimeList readPlateTimeFile(FILE *f)
{
    PlateTimeList list = NULL;
    Plate plate;
    while (readField(plate, SEPARATOR, f) > 0)
    {
        PlateTime currentPlateTime;
        char timeString[9];
        Time time;

        if (readField(timeString, SEPARATOR, f) == 0)
        {
            exit(1);
        }

        time = stringToTime(timeString);
        strcpy(currentPlateTime.plate, plate);
        currentPlateTime.time = time;
        list = addPlateTime(list, currentPlateTime);
    }
    return list;
}

int readField(char buffer[], char sep, FILE *f)
{
    int i = 0;
    char ch = fgetc(f);
    while (ch != sep && ch != 10 && ch != EOF)
    {
        buffer[i++] = ch;
        ch = fgetc(f);
    }
    buffer[i] = '\0';
    return i;
}

PlateTimeList readGateFiles(GateName name)
{
    PlateTimeList list = NULL;
    int i = 1;
    FILE *gateFile;
    do
    {
        char gateFileName[30];
        sprintf(gateFileName, "%s-%d.txt", name, i);
        gateFile = fopen(gateFileName, "r");
        if (gateFile != NULL)
        {
            PlateTimeList currentGateList = readPlateTimeFile(gateFile);
            fclose(gateFile);
            list = concatPlateTimeLists(list, currentGateList);
        }
        i++;
    }
    while (gateFile != NULL);
    return list;
}
